//
//  AuthNService.m
//
//  Created by CA Technologies
//  Copyright (c) 2012 CA Technologies. All rights reserved.
//

#import "AuthNService.h"

#define kuser @"user"
#define kpassword @"password"
#define kUrlConnTimeoutKey @"urltimeout"
#define kdefaultConnTimeout 10.0
#define kMaxURLLEN 512
static char *alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

@interface AuthNService (private)
- (void) addHeaderField:(NSMutableURLRequest *)req;
@end

@implementation AuthNService

@synthesize nwErrMsg;
@synthesize cookieStr;
@synthesize authNContext;

- (id) init {
    if (self = [super init])
    {
        NSLog(@"Initializing AuthNService v22");
        self.nwErrMsg = nil;
        self.cookieStr = nil;
    }
    return self;
}

- (void) dealloc {
    self.nwErrMsg = nil;
    self.cookieStr = nil;
    self.authNContext = nil;
}

- (AuthNContext *) authNContext {
    if (!authNContext)
    {
        authNContext = [[AuthNContext alloc] init];
    }
    return authNContext;
}

- (void) connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
    NSLog(@"connection:didFailWithError:");
    authNContext.headers = [[NSMutableDictionary alloc] initWithDictionary:error.userInfo];
    authNContext.body = error.localizedDescription;
    authNContext.failureBlock();
}

- (void) connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    NSLog(@"-----------> connection:didReceiveData:");
    NSString *rsltStr = nil;
    if (data != nil)
    {
        rsltStr = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
        rsltStr = [rsltStr stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
        if ( rsltStr == nil || ([rsltStr length] <= 0) )
        {
            self.nwErrMsg = NSLocalizedString(@"Unknown Network Error", @"Unknown Network Error");
            rsltStr = nil;
        }
        
        NSLog(@"result from server len %d",[rsltStr length]);
    }
    else
    {
        self.nwErrMsg = NSLocalizedString(@"Unknown Network Error", @"Unknown Network Error");
        NSLog(@"submited request, got nil result");
    }
    
    if (authNContext.body == nil)
    {
        authNContext.body = [[NSString alloc] init];
    }
    
    NSMutableString *mutableString = [NSMutableString stringWithString:authNContext.body];
    [mutableString appendString:rsltStr];
    authNContext.body = mutableString;
}

- (void) connection:(NSURLConnection *)connection didReceiveResponse:(NSHTTPURLResponse *)response {
    NSLog(@"connection:didReceiveResponse:");
    authNContext.headers = [[NSMutableDictionary alloc] initWithDictionary:response.allHeaderFields];
    authNContext.statusCode = response.statusCode;
    authNContext.urlStr = response.URL.description;
    
    if (authNContext.processCookies) {
        /* Get an array with all the cookies */
        NSArray *cookies = [NSHTTPCookie cookiesWithResponseHeaderFields:response.allHeaderFields
                                                                  forURL:response.URL];
        NSLog(@"RECEIVED COOKIES %i", cookies.count);
        // print out the cookie values
        for (NSHTTPCookie *cookie in cookies)
        {
            NSLog(@"nName: %@ nLen: %i nValue: %@ nExpires: %@", [cookie name], [[cookie value] length], [cookie value], [cookie expiresDate]);
        }
        
        /* Add the array of cookies in the shared cookie storage instance */
        [[NSHTTPCookieStorage sharedHTTPCookieStorage]
         setCookies:cookies
         forURL:response.URL
         mainDocumentURL:nil];
    }
}


- (NSURLRequest *) connection:(NSURLConnection *)connection willSendRequest:(NSURLRequest *)request redirectResponse:(NSHTTPURLResponse *)response
{
    NSLog(@"connection:willSendRequest:redirectResponse:");
    if (response != nil)
    {
        [self connection:connection didReceiveResponse:response];
    }
    return request;
}


- (void) connectionDidFinishLoading:(NSURLConnection *)connection
{
    authNContext.successBlock();
}

- (AuthNContext *) execute {
    NSLog(@"----------------- INIT %@",authNContext.urlStr);
    NSString *urlMod = [NSString stringWithFormat:@"%@",authNContext.urlStr];
    
    // set query auth if authNContext uid/pw have been specified, these will be placed in query string as ?user=aValue&password=aValue
    if ( authNContext.authType == AUTHN_AUTH_QUERY)
    {
        NSLog(@"Processing authNContext.queryAuth");
        NSLog(@"using uid=%@ pwd=%@",authNContext.uid,authNContext.pw);
        NSString *uid = authNContext.uid;
        NSString *pw = authNContext.pw;
        if (uid)
        {
            NSLog(@"Added authNContext.uid to queryVariablesDict");
            [authNContext addToQueryValuesTheKey:kuser andValue:uid];
        }
        
        if (pw)
        {
            NSLog(@"Added authNContext.pw to queryVariablesDict");
            [authNContext addToQueryValuesTheKey:kpassword andValue:pw];
        }
    }
    
    // add any query parameters into the url
    int i = 0;
    for (id key in authNContext.queryValues)
    {
        if (i == 0)
        {
            urlMod = [urlMod stringByAppendingString:@"?"];
        }
        else
        {
            urlMod = [urlMod stringByAppendingString:@"&"];
        }
        
        urlMod = [urlMod stringByAppendingString:[NSString stringWithFormat:@"%@=%@",key,[authNContext.queryValues objectForKey:key]]];
        i++;
    }
    
    NSLog(@"finished composing request: %@",urlMod);
    
    NSURL *url = [NSURL URLWithString:[NSString stringWithFormat:@"%@",urlMod]];
    
    double timeoutval = [[NSUserDefaults standardUserDefaults] doubleForKey:kUrlConnTimeoutKey];
    if (timeoutval <= 0)
    {
        timeoutval = kdefaultConnTimeout;
    }
    
    NSMutableURLRequest *req = [NSMutableURLRequest requestWithURL:url cachePolicy:NSURLCacheStorageAllowedInMemoryOnly timeoutInterval:timeoutval];
    
    if (authNContext.processCookies) {
        // add the cookies back in
        [[NSHTTPCookieStorage sharedHTTPCookieStorage] setCookieAcceptPolicy: NSHTTPCookieAcceptPolicyAlways];
        NSArray *currentCookies = [[NSHTTPCookieStorage sharedHTTPCookieStorage] cookiesForURL:url];
        NSDictionary *headers = [NSHTTPCookie requestHeaderFieldsWithCookies:currentCookies];
        [req setAllHTTPHeaderFields:headers];
        NSLog(@"SENDING COOKIES %i", currentCookies.count);
        // print out the cookie values
        for (NSHTTPCookie *cookie in currentCookies)
        {
            NSLog(@"nName: %@ nLen: %i nValue: %@ nExpires: %@", [cookie name], [[cookie value] length], [cookie value], [cookie expiresDate]);
        }
    }
    else {
        [[NSHTTPCookieStorage sharedHTTPCookieStorage] setCookieAcceptPolicy: NSHTTPCookieAcceptPolicyNever];
    }
    
    // set basic auth if authNContext uid/pw have been specified
    if ( authNContext.authType == AUTHN_AUTH_BASIC) {
        NSLog(@"Processing authNContext.basicAuth");
        NSString *uid = authNContext.uid;
        NSString *pw = authNContext.pw;
        if ( uid != nil && (pw != nil) )
        {
            NSString *basicAuthString = [AuthNService basicAuthStringForUID:uid pw:pw];
            [req addValue:basicAuthString forHTTPHeaderField:@"Authorization"];
        }
    }
    
    // add in any additional headers that have been provided
    for (id key in authNContext.headers) {
        [req setValue:[authNContext.headers objectForKey:key] forHTTPHeaderField:key];
    }
    
    // set a ca specific header
    //[self addHeaderField:req];
    
    // lets set the default to be GET
    [req setHTTPMethod:AUTHN_HTTP_GET];
    
    // if a body has been provided, lets assume this is a POST vs. GET
    if (authNContext.body) {
        NSLog(@"Setting up POST %@",authNContext.body);
        NSLog(@"using uid=%@ pwd=%@",authNContext.uid,authNContext.pw);
        NSData *data = [authNContext.body dataUsingEncoding:NSUTF8StringEncoding];
        [req setHTTPBody:data];
        [req setHTTPMethod:AUTHN_HTTP_POST];
    }
    
    // lets see if an override has been sent through, wo ho!
    if (authNContext.httpMethodOverride) {
        [req setHTTPMethod:authNContext.httpMethodOverride];
    }
    
    NSLog(@"SENDING %@ - timeout: (%f secs) headers: %@",url.description,timeoutval,req.allHTTPHeaderFields);
    [NSURLConnection connectionWithRequest:req delegate:self];
    return authNContext;
}


- (void) addHeaderField:(NSMutableURLRequest *)req
{
    NSString *appver = [[NSUserDefaults standardUserDefaults] stringForKey:@"1.0"];
    if (appver == nil)
    {
        appver = @"ca-iphone/1.0";
    }
    else
    {
        appver = [NSString stringWithFormat:@"ca-iphone/%@",appver];
    }
    
    [req setValue:appver forHTTPHeaderField:@"x-ca-appid"];
}


+ (NSString *) escapeUrlString:(NSString *)myStr
{
    //NSLog(@"Received str %@",myStr);
    
    NSMutableString *escaped = [[NSMutableString alloc] initWithString:[myStr stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]];
    //NSLog(@"length of original string %d",[myStr length]);
    //NSLog(@"escaped string %@",escaped);
    [escaped replaceOccurrencesOfString:@"&" withString:@"%26" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    [escaped replaceOccurrencesOfString:@"+" withString:@"%2B" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    [escaped replaceOccurrencesOfString:@"," withString:@"%2C" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    [escaped replaceOccurrencesOfString:@"/" withString:@"%2F" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    [escaped replaceOccurrencesOfString:@":" withString:@"%3A" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    [escaped replaceOccurrencesOfString:@";" withString:@"%3B" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    [escaped replaceOccurrencesOfString:@"=" withString:@"%3D" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    [escaped replaceOccurrencesOfString:@"?" withString:@"%3F" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    [escaped replaceOccurrencesOfString:@"@" withString:@"%40" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    [escaped replaceOccurrencesOfString:@" " withString:@"%20" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    [escaped replaceOccurrencesOfString:@"\t" withString:@"%09" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    [escaped replaceOccurrencesOfString:@"#" withString:@"%23" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    [escaped replaceOccurrencesOfString:@"<" withString:@"%3C" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    [escaped replaceOccurrencesOfString:@">" withString:@"%3E" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    [escaped replaceOccurrencesOfString:@"\"" withString:@"%22" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    [escaped replaceOccurrencesOfString:@"\n" withString:@"%0A" options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    [escaped replaceOccurrencesOfString:@"." withString:@"%2E"options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    //[escaped replaceOccurrencesOfString:@"%" withString :@"%25"options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    //[escaped replaceOccurrencesOfString:@"{" withString :@"%7B"options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    //[escaped replaceOccurrencesOfString:@"}" withString :@"%7D"options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    //[escaped replaceOccurrencesOfString:@"|" withString :@"%7C"options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    //[escaped replaceOccurrencesOfString:@"(" withString :@"%28"options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    //[escaped replaceOccurrencesOfString:@")" withString :@"%29"options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    //[escaped replaceOccurrencesOfString:@"^" withString :@"%5E"options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    //[escaped replaceOccurrencesOfString:@"`" withString :@"%60"options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    
    //NSRange rr = [escaped rangeOfString:@"?"];
    
    //NSLog(@"? exist at %d\n",rr.location);
    
    //NSLog(@"after escaping string %@",escaped);
    
    NSString *retStr = [NSString stringWithString:escaped];
    
    return retStr;
}


+ (NSString *) unescapeUrlString:(NSString *)myStr
{
    if (myStr == nil)
    {
        return nil;
    }
    
    NSMutableString *escaped = [[NSMutableString alloc] initWithString:[myStr stringByReplacingPercentEscapesUsingEncoding:NSUTF8StringEncoding]];
    [escaped replaceOccurrencesOfString:@"+" withString:@" " options:NSCaseInsensitiveSearch range:NSMakeRange(0, [escaped length])];
    NSString *retStr = [NSString stringWithString:escaped];
    
    return retStr;
}


+ (NSString *) basicAuthStringForUID:(NSString *)uid pw:(NSString *)pw
{
    // create a plaintext string in the format username:password
    NSString *loginString = [NSString stringWithFormat:@"%@:%@", uid, pw];
    // employ the Base64 encoding above to encode the authentication tokens
    NSString *encodedLoginData = [AuthNService base64encode:[loginString dataUsingEncoding:NSUTF8StringEncoding]];
    // create the contents of the header
    NSString *authHeader = [@"Basic " stringByAppendingFormat:@"%@", encodedLoginData];
    return authHeader;
}


+ (NSString *) base64encode:(NSData *)plainText
{
    int encodedLength = ( 4 * ( ([plainText length] / 3) + (1 - ( 3 - ([plainText length] % 3) ) / 3) ) ) + 1;
    unsigned char *outputBuffer = malloc(encodedLength);
    
    memset(outputBuffer, 0, encodedLength);
    
    unsigned char *inputBuffer = (unsigned char *)[plainText bytes];
    
    NSInteger i;
    NSInteger j = 0;
    int remain;
    
    for (i = 0; i < [plainText length]; i += 3)
    {
        remain = [plainText length] - i;
        
        outputBuffer[j++] = alphabet[(inputBuffer[i] & 0xFC) >> 2];
        outputBuffer[j++] = alphabet[( (inputBuffer[i] & 0x03) << 4 ) |
                                     ( (remain > 1) ? ( (inputBuffer[i + 1] & 0xF0) >> 4 ) : 0 )];
        
        if (remain > 1)
        {
            outputBuffer[j++] = alphabet[( (inputBuffer[i + 1] & 0x0F) << 2 )
                                         | ( (remain > 2) ? ( (inputBuffer[i + 2] & 0xC0) >> 6 ) : 0 )];
        }
        else
        {
            outputBuffer[j++] = '=';
        }
        
        if (remain > 2)
        {
            outputBuffer[j++] = alphabet[inputBuffer[i + 2] & 0x3F];
        }
        else
        {
            outputBuffer[j++] = '=';
        }
    }
    
    outputBuffer[j] = 0;
    NSInteger outlen = j;
    
    NSString *result = [NSString stringWithCharacters:(unichar *)outputBuffer length:outlen];
    free(outputBuffer);
    
    return result;
}


@end
