package com.example.smandroidauthn;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.annotation.TargetApi;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.webkit.CookieManager;
import android.webkit.CookieSyncManager;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

/**
 * Activity which displays a login screen to the user, offering registration as
 * well.
 */
public class LoginActivity extends Activity {

	/**
	 * The default email to populate the email field with.
	 */
	public static final String EXTRA_EMAIL = "com.example.android.authenticatordemo.extra.EMAIL";

	/**
	 * Keep track of the login task to ensure we can cancel it if requested.
	 * There are two background tasks here "UserLoginTask" for main async login
	 * and AuthenticatedTask which checks to ensure the user is authenticated.
	 */
	private UserLoginTask mAuthTask = null;
	private AuthenticatedTask mCheckAuth = null;

	// Values for email and password at the time of the login attempt.
	private String mUserID;
	private String mPassword;

	// UI references.
	private EditText mUserIDView;
	private EditText mPasswordView;
	private View mLoginFormView;
	private View mLoginStatusView;
	private TextView mLoginStatusMessageView;
	
	/*CookieManager is needed to pass on the cookies to any WebView that
	 * is used later on.
	 */
    CookieManager cookieManager = null;
    
    private boolean showBizPage;
    private String targetURL;
    private String loginID;
    private String serverURL;
    private String loginPath;

    
    /**
     * Main method for app launches here
     * Sets up the Login page and all the listeners that are needed
     * for the buttons.
     */
    
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		loadConfig();
		
		setContentView(R.layout.activity_login);
		
		CookieSyncManager.createInstance(this); 
		cookieManager = CookieManager.getInstance();
		
		// Set up the login form.
		mUserID = getIntent().getStringExtra(EXTRA_EMAIL);
		mUserIDView = (EditText) findViewById(R.id.userid);
		
	
		if(loginID.length() > 0 && ( mUserID == null || mUserID.length() > 0 )){
			mUserIDView.setText(loginID);
		}else{
			mUserIDView.setText(mUserID);
		}

		mPasswordView = (EditText) findViewById(R.id.password);
		mPasswordView.setOnEditorActionListener(new TextView.OnEditorActionListener() {
					@Override
					public boolean onEditorAction(TextView textView, int id,
							KeyEvent keyEvent) {
						if (id == R.id.login || id == EditorInfo.IME_NULL) {
							attemptLogin();
							return true;
						}
						return false;
					}
				});

		mLoginFormView = findViewById(R.id.login_form);
		mLoginStatusView = findViewById(R.id.login_status);
		mLoginStatusMessageView = (TextView) findViewById(R.id.login_status_message);

		findViewById(R.id.sign_in_button).setOnClickListener(
				new View.OnClickListener() {
					@Override
					public void onClick(View view) {
						attemptLogin();
					}
				}
		);
		
		findViewById(R.id.loginStatus).setOnClickListener(
				new View.OnClickListener() {
					@Override
					public void onClick(View v) {
						attemptAuthenticated();
					}
				}
		);
        
	}
	
	/*
	 * Shows the webView once we are logged in. 
	 */
	public void showWebView(){
		Intent intent = new Intent(this,WvActivity.class);
		intent.putExtra("url", targetURL.replaceFirst("authn:", ""));
		startActivity(intent);
	}

	/*
	 * Setup the menu button menu. Will bring up the config menu.
	 * @see android.app.Activity#onCreateOptionsMenu(android.view.Menu)
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		getMenuInflater().inflate(R.menu.activity_login, menu);
		return true;
	}
	
	/*
	 * Starts a task to check to see if the user is authenticated.
	 */
	public void attemptAuthenticated(){
		if(mCheckAuth != null){
			return;
		}
		mCheckAuth = new AuthenticatedTask();
		mCheckAuth.execute((Void) null);
	}

	/**
	 * Attempts to sign in or register the account specified by the login form.
	 * If there are form errors (invalid email, missing fields, etc.), the
	 * errors are presented and no actual login attempt is made.
	 */
	public void attemptLogin() {
		if (mAuthTask != null) {
			return;
		}
		loadConfig();
		
		// Reset error
		mUserIDView.setError(null);
		mPasswordView.setError(null);

		// Store values at the time of the login attempt.
		mUserID = mUserIDView.getText().toString();
		mPassword = mPasswordView.getText().toString();

		boolean cancel = false;
		View focusView = null;

		// Check for a valid password. insert your own password reqs here
		if (TextUtils.isEmpty(mPassword)) {
			mPasswordView.setError(getString(R.string.error_field_required));
			focusView = mPasswordView;
			cancel = true;
		} else if (mPassword.length() < 4) {
			mPasswordView.setError(getString(R.string.error_invalid_password));
			focusView = mPasswordView;
			cancel = true;
		}
		
		//Check to ensure the serverURL and LoginPath are set
		if(serverURL == null || serverURL.length() <= 0){
			Log.d("smandroidauthn", "Show popup!");
			showErrorMessage(getString(R.string.login_config_required));
			return;
		}
		if(loginPath == null || loginPath.length() <= 0){
			
			showErrorMessage(getString(R.string.login_config_required));
			return;
		}
		

		// Check for a valid email address.
		if (TextUtils.isEmpty(mUserID)) {
			mUserIDView.setError(getString(R.string.error_field_required));
			focusView = mUserIDView;
			cancel = true;
		} 

		if (cancel) {
			// There was an error; don't attempt login and focus the first
			// form field with an error.
			focusView.requestFocus();
		} else {
			// Show a progress spinner, and kick off a background task to
			// perform the user login attempt.
			mLoginStatusMessageView.setText(R.string.login_progress_signing_in);
			showProgress(true);
			mAuthTask = new UserLoginTask();
			mAuthTask.execute((Void) null);
		}
	}
	
	/*
	 * Shows an error message on the screen in a Dialog Box
	 */
	private void showErrorMessage(String errorMessage){
		Builder alertBuilder = new AlertDialog.Builder(this);
		alertBuilder.setMessage(errorMessage);
		alertBuilder.setNeutralButton(R.string.ok, new DialogInterface.OnClickListener(){

			@Override
			public void onClick(DialogInterface dialog, int which) {
				return;
			}
		});
		alertBuilder.show();
	}

	/**
	 * Shows the progress UI and hides the login form.
	 */
	@TargetApi(Build.VERSION_CODES.HONEYCOMB_MR2)
	private void showProgress(final boolean show) {
		// On Honeycomb MR2 we have the ViewPropertyAnimator APIs, which allow
		// for very easy animations. If available, use these APIs to fade-in
		// the progress spinner.
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB_MR2) {
			int shortAnimTime = getResources().getInteger(
					android.R.integer.config_shortAnimTime);

			mLoginStatusView.setVisibility(View.VISIBLE);
			mLoginStatusView.animate().setDuration(shortAnimTime)
					.alpha(show ? 1 : 0)
					.setListener(new AnimatorListenerAdapter() {
						@Override
						public void onAnimationEnd(Animator animation) {
							mLoginStatusView.setVisibility(show ? View.VISIBLE
									: View.GONE);
						}
					});

			mLoginFormView.setVisibility(View.VISIBLE);
			mLoginFormView.animate().setDuration(shortAnimTime)
					.alpha(show ? 0 : 1)
					.setListener(new AnimatorListenerAdapter() {
						@Override
						public void onAnimationEnd(Animator animation) {
							mLoginFormView.setVisibility(show ? View.GONE
									: View.VISIBLE);
						}
					});
		} else {
			// The ViewPropertyAnimator APIs are not available, so simply show
			// and hide the relevant UI components.
			mLoginStatusView.setVisibility(show ? View.VISIBLE : View.GONE);
			mLoginFormView.setVisibility(show ? View.GONE : View.VISIBLE);
		}
	}

	/*
	 * Start the config view
	 */
	public void doConfigView(){
		Intent intent = new Intent(this, ConfView.class);
		startActivity(intent);
	}
	
	/*
	 * Load the config variables
	 */
	private void loadConfig(){
		SharedPreferences preferences = getSharedPreferences(ConfView.PREFERENCES_NS, Context.MODE_PRIVATE);
		targetURL = (preferences.getString(ConfView.TARGET_URL,""));
		loginID = (preferences.getString(ConfView.LOGIN_ID,""));
		serverURL = (preferences.getString(ConfView.SERVER_URL,""));
		loginPath = (preferences.getString(ConfView.LOGIN_PATH,""));
		showBizPage = (preferences.getBoolean(ConfView.SHOW_BIZPAGE,true));
	}
	
	/*
	 * Start the config view when the config menu item is selected
	 */
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch(item.getItemId()){
			case R.id.config_menu_item:
				doConfigView();
				return true;
			default:
				return super.onOptionsItemSelected(item);
		}
	}
	
	/**
	 * Represents an asynchronous login task used to authenticate
	 * the user.
	 */
	public class UserLoginTask extends AsyncTask<Void, Void, Boolean> {
		Exception caughtException=null;
		StringBuffer result = new StringBuffer();
		
		//This is called when the AsyncTask is executed
		@Override
		protected Boolean doInBackground(Void... params) {
			
			loadConfig();
		
			if(mPassword == null || mPassword.length() <= 0){
				return false;
			}
			
			AuthComm auth1 = new AuthComm(getSharedPreferences(ConfView.PREFERENCES_NS, Context.MODE_PRIVATE));
			
			try {
				boolean res = auth1.login(mUserID, mPassword, result);
				return res;
			} catch(Exception e){
				//Save aside the exception so that we can process it in the PostExecute
				caughtException=e;
				return false;
			}
		}

		//When the background task returns, we head here.
		@Override
		protected void onPostExecute(final Boolean success) {
			mAuthTask = null;
			showProgress(false);
			TextView v1 = (TextView)findViewById(R.id.StatusTextView);
			v1.setText(result.toString());
			if (success) {
				if(showBizPage){
					showWebView();
				} else {
					Context context = getApplicationContext();
					Toast toast = Toast.makeText(context,getString(R.string.successful_login),Toast.LENGTH_SHORT);
					toast.show();
				}
			} else {
				if(caughtException!=null){
					Context context = getApplicationContext();
					Toast toast = Toast.makeText(context,caughtException.getLocalizedMessage(),Toast.LENGTH_LONG);
					toast.show();
					v1.setText("");
				} else {
					mPasswordView.setError(getString(R.string.error_incorrect_password));
					mPasswordView.requestFocus();
					v1.setText("");
				}
			}
		}

		@Override
		protected void onCancelled() {
			mAuthTask = null;
			showProgress(false);
		}
	}
	
	/*
	 * The background AsyncTask for checking the status of the authentication session
	 */
	public class AuthenticatedTask extends AsyncTask<Void, Void, Boolean> {
    	Exception caughtException=null;
    	
		@Override
		protected Boolean doInBackground(Void... params) {
			AuthComm comm = new AuthComm(getSharedPreferences(ConfView.PREFERENCES_NS, Context.MODE_PRIVATE));
			try {
				return comm.isAuthenticated();
			} catch(Exception e){
				caughtException=e;
				return false;
			}
		}
		
		@Override
		protected void onPostExecute(final Boolean success){
			mCheckAuth = null;
			Log.d("smandroidauthn", "In Post execute on AuthenticatedTask");
			Context context = getApplicationContext();
			Toast toast = null;
			TextView v1 = (TextView)findViewById(R.id.StatusTextView);
			if(success){
				toast = Toast.makeText(context,getString(R.string.authenticated),Toast.LENGTH_SHORT);
				v1.append("\n" + getString(R.string.authenticated));
			}else{
				v1.append("\n" + getString(R.string.not_authenticated));
				if(caughtException!=null){
					toast = Toast.makeText(context,caughtException.getLocalizedMessage(),Toast.LENGTH_LONG);
				} else {
					toast = Toast.makeText(context,getString(R.string.not_authenticated),Toast.LENGTH_SHORT);
				}
			}
			toast.show();
		}
		
		protected void onCancelled(){
			mCheckAuth = null;
		}
    	
    }
	
}
